#include <cassert>
#include <iostream>
#include <vector>
#include <random>
#include <set>
#include <algorithm>
#include <string>
#include <map>
#include <sstream>
#include <regex>
#include <deque>
#include <numeric>
#include <fstream>
#include <queue>
#include <unordered_map>

constexpr int64_t p1 = 1000000007;
constexpr int64_t p2 = 1000000009;

int64_t modpow(int64_t x, int64_t n, int64_t p)
{
    int64_t result = 1;
    while (n > 0)
    {
        if (n % 2)
            result = (result * x) % p;
        x = (x * x) % p;
        n /= 2;
    }
    return result;
}

int64_t modinv(int64_t x, int64_t p)
{
  return modpow(x, p - 2, p);
}

int main()
{    
    int64_t n, k;
    std::cin >> n >> k;

    std::vector<int64_t> vitality(n);
    for (int i = 0; i < n; i++)
        std::cin >> vitality[i];

    int64_t iter = 0;
    int firstnonzero = 0;
    while (iter < k)
    {
        std::sort(vitality.begin() + firstnonzero, vitality.end());
        while (firstnonzero < n && vitality[firstnonzero] == 0)
            firstnonzero++;
        int nonzero = n - firstnonzero;
        if (nonzero < 2)
            break;

        int64_t biggeststep = k - iter;
        for (int i = firstnonzero; i < n - 1; i++)
        {
            biggeststep = std::min(biggeststep, vitality[i + 1] / vitality[i]);
        }
        
        // a big step is worth it if nonzero < biggeststep
        if (nonzero < biggeststep)
        {
            std::vector<int64_t> binoms1(nonzero);
    	    std::vector<int64_t> binoms2(nonzero);
            binoms1[0] = 1;
	        binoms2[0] = 1;
            for (int i = 1; i < nonzero; i++)
            {
                int64_t val1 = (binoms1[i - 1] * (biggeststep + i - 1)) % p1;
                binoms1[i] = (val1 * modinv(i,p1)) % p1;
        		int64_t val2 = (binoms2[i - 1] * (biggeststep + i - 1)) % p2;
	        	binoms2[i] = (val2 * modinv(i,p2)) % p2;
            }

            std::vector<int64_t> newterms(nonzero);
            for (int i = 0; i < nonzero; i++)
            {
                int64_t newval1 = 0;
        		int64_t newval2 = 0;
                int64_t sign = 1;
                for (int j = 0; j <= i; j++)
                {
        		  int64_t term1 = (sign * binoms1[j] * (vitality[firstnonzero + i - j] % p1)) % p1;
		          int64_t term2 = (sign * binoms2[j] * (vitality[firstnonzero + i - j] % p2)) % p2;
        		  newval1 = (p1 + newval1 + term1) % p1;
        		  newval2 = (p2 + newval2 + term2) % p2;
		          sign *= -1;
                }
        		// CRT
        		int64_t k = (newval2 + p2 - newval1) % p2;
		        k = (k * modinv(p1, p2)) %  p2;		
                newterms[i] = newval1 + k * p1;
            }

            for (int i = 0; i < nonzero; i++)
                vitality[firstnonzero + i] = newterms[i];

            iter += biggeststep;
        }
        else
        {
            for (int i = firstnonzero; i < n - 1; i++)
            {
                vitality[i + 1] -= vitality[i];
            }
            iter++;
        }
    }

    for (int i = 0; i < n; i++)
        std::cout << vitality[i] << " ";
    std::cout << std::endl;
}
